$def with ()
$var title: Syndication

<form method=post>
Twitter<br>
<input name=twitter_username placeholder=username><br>
<input name=twitter_password placeholder=password><br>
GitHub<br>
<input name=github_username placeholder=username><br>
<input name=github_token placeholder=token><br>
<button>Save</button>
</form>
